/*
 * 
 Developed on 25/2/20016
 by Abdel-Razzak Merheb
 Flying wing - Airplane Remote control using
 433 MHz RF Module Transmitter
 (You need to choose between flying wing and airplane BEFORE start controlling)
 RF-Xmit-Demo-1.ino
 Base transmission code is developed by: 
  DroneBot Workshop 2018
  https://dronebotworkshop.com

I am using here the internal pull-up resistors for the switches  

NO trimming push buttons!!!!!!!!!!!!!!!!!

VCC – This can be any positive DC voltage from 3 to 12 volts. In this experiment we will get 5 volts from our Arduino.
GND – The ground connection, connected to one of the Arduino ground terminals.
DATA IN – This is connected to pin 12 of the Arduino. You should try and use pin 12 as by default the library we’ll be using in our sketch uses this pin for data output.  

There are two joysticks with 4 potentiometer, a trimming switch, and a landing gear switch
When the user wants to trim the Elevator, Rudder, and Aileron, s/he pushes the switch to TRIM, and then moves the relevant potentiometer to trim the desired variable !!!!!!!!!!!!!
*
*/

// ABD: Transmitting potentiometer value through 433 MHz 4/6/2021
 
// Include RadioHead Amplitude Shift Keying Library
#include <RH_ASK.h>
// Include dependant SPI Library 
#include <SPI.h> 
 
// Create Amplitude Shift Keying Object
RH_ASK rf_driver;

// Select the potentiometer pins
int ThrottlePin = A2;    // select the input pin for the thrust potentiometer
int RudderPin = A3;    // select the input pin for the rudder potentiometer
int ElevatorPin = A4;    // select the input pin for the elevator potentiometer
int AileronPin = A5;    // select the input pin for the aileron potentiometer
// Switches/pusbuttons' pins
const int GearPin = 2;     // Landing gear pushbutton is pin 2
const int TrimmerSwitchPin = 3;     // Trimmer switch is pin 3
const int AFSwitchPin = 4;     // Airplane/flying wing switch is pin 4
const int BuzzPin = 5;   // The buzzer pin

// Control variables as a structure
struct controlData{
int Throttle;     // The Throttle variable
int Rudder;     // The Rudder variable
int Elevator;     // The Elevator variable
int Aileron;     // The Aileron variable
int GearsDown;  // The gear down switch (initially fears are down -> active high)
int AirplaneFlyingWing;   // The variable: airplane OR flying wing?
int ElevTrim = 0;  // The ELEVATOR trimmer value
int RudTrim = 0;  // The RUDDER trimmer value
int AilTrim = 0;  // The Aileron trimmer value
}myData;

// Trimming variables (Not in the structure)
int ElevTrimmer = 0;  // The ELEVATOR trimmer value
int RudTrimmer = 0;  // The RUDDER trimmer value
int AilTrimmer = 0;  // The Aileron trimmer value

// Trimming switch
int TrimmerSwitch;  // Trimmer switch: you can trim ONLY if this switch is ON
// Airplane/flying wing switch
int AFSwitch;  // Airplane/flying wing switch: you need to choose BEFORE start controlling


byte tx_buf[sizeof(myData)];
// byte tx_buf[sizeof(myData)] = {0};


void setup()
{
    // initialize the pushbutton and switch pins as an inputs
    // Internal pull-up resistors are used /////////////////////////////////////////////////////////////////
    pinMode(GearPin, INPUT_PULLUP);    // GearsDown switch connected to TrimmerSwitchPin is an input
    pinMode(TrimmerSwitchPin, INPUT_PULLUP);    // Trimmer switch connected to GearPin is an input
    pinMode(AFSwitch, INPUT_PULLUP);    // Airplane/flying wing switch connected to GearPin is an input ///////// However, it didn't work for pin 4!!! we have to connect a pull-up resistor to pin 4!!!!
    pinMode(BuzzPin, OUTPUT);    //  The buzzer pin is an output
    // Initialize ASK Object
    rf_driver.init();
    // Setup Serial Monitor
    Serial.begin(9600);    
}


void Buzz() { // Sound the buzzer
  // turn on the buzzer
  digitalWrite(BuzzPin, HIGH);
  delay(100);
  // turn off the buzzer
  digitalWrite(BuzzPin, LOW);
}


void loop()
{

    
  // The structure is opened for change
  struct controlData myData;

  // Check from the beginning: Airplane OR flying wing???
  myData.AirplaneFlyingWing = digitalRead(AFSwitchPin);   // Read the state of the AFSwitchPin switch

  // Load the structure trim values (to be sent)
  myData.ElevTrim = ElevTrimmer;  // The ELEVATOR trimmer value
  myData.RudTrim = RudTrimmer;  // The Rudder trimmer value
  myData.AilTrim = AilTrimmer;  // The Aileron trimmer value


  // read the state of the pushbuttons/switches
   myData.GearsDown = digitalRead(GearPin);   // Read the state of the landing gear switch
   
  // read the values from the potentiometer joysticks
  // myData.Throttle = map(analogRead(ThrottlePin),0,1023,-1,1);     // reads the value of the potentiometer (value between -1 and 1)
  myData.Throttle = map(analogRead(ThrottlePin),0,1023,0,100);     // Read the Throttle potentiometer
  myData.Rudder = map(analogRead(RudderPin),0,1023,0,100);     // Read the Rudder potentiometer
  myData.Elevator = map(analogRead(ElevatorPin),0,1023,0,100);     // Read the Elevator potentiometer
  myData.Aileron = map(analogRead(AileronPin),0,1023,0,100);     // Read the Aileron potentiometer


    // If the trimmer switch is ON, then go to trimming
  TrimmerSwitch = digitalRead(TrimmerSwitchPin);   // Read the state of the Trimmer switch
    if (TrimmerSwitch == HIGH)
      {
        Serial.print("Trimming"); // A message telling that you are in the trimming mode
        // Change the Elevator trim value
        if (map(analogRead(ElevatorPin),0,1023,0,100) > 70) // If the Elevator pot is set to maximum, increase the trim
          {
            ElevTrimmer ++; // Increase the elevator trim
            if(ElevTrimmer >= 50) ElevTrimmer = 50; // Max trim value is 50
            Buzz(); // Sound the buzzer
            // Wait until the user removes his/her hand from the pushbutton
          }
        if (map(analogRead(ElevatorPin),0,1023,0,100) < 30)  // If the Elevator pot is set to minimum, decrease the trim
          {
            ElevTrimmer --; // Decrease the elevator trim
            if(ElevTrimmer <= 0) ElevTrimmer = 0; // Min trim value is 0
            Buzz(); // Sound the buzzer
          }
          
        // Change the Rudder trim value
        if (map(analogRead(RudderPin),0,1023,0,100) > 70) // If the Rudder pot is set to maximum, increase the trim
          {
            RudTrimmer ++; // Increase the rudder trim
            if(RudTrimmer >= 50) RudTrimmer = 50; // Max trim value is 50
            Buzz(); // Sound the buzzer
          }
                  
        if (map(analogRead(RudderPin),0,1023,0,100) < 30)  // If the Rudder pot is set to minimum, decrease the trim
          {
            RudTrimmer --; // Decrease the rudder trim
            if(RudTrimmer <= 0) RudTrimmer = 0; // Min trim value is 0
            Buzz(); // Sound the buzzer
          }
                  
        // Change the Aileron trim value
        if (map(analogRead(AileronPin),0,1023,0,100) > 70) // If the Aileron pot is set to maximum, increase the trim
          {
            AilTrimmer ++; // Increase the aileron trim
            if(AilTrimmer >= 50) AilTrimmer = 50; // Max trim value is 50
            Buzz(); // Sound the buzzer
          }
                  
        if (map(analogRead(AileronPin),0,1023,0,100) < 30)  // If the Aileron pot is set to minimum, decrease the trim
          {
            AilTrimmer --; // Decrease the aileron trim
            if(AilTrimmer <= 0) AilTrimmer = 0; // Min trim value is 0
            Buzz(); // Sound the buzzer
          }   
            // While trimming, all the values are zero (throttle at the lowest and servos at the middle)     
            myData.Throttle = 0;     // Minimum throttle
            myData.Rudder = 50;     // Mid Rudder 
            myData.Elevator = 50;     // Mid Elevator
            myData.Aileron = 50;     // Mid Aileron
      } // For the trimming if
      

      // Potentiometer values to be sent
      Serial.print("Throttle value: ");
      Serial.println(myData.Throttle); 
      Serial.print("Rudder value: ");
      Serial.println(myData.Rudder); 
      Serial.print("Elevator value: ");
      Serial.println(myData.Elevator); 
      Serial.print("Aileron value: ");
      Serial.println(myData.Aileron);  
      Serial.print("Gears value: ");
      Serial.println(myData.GearsDown);  
      Serial.print("Elevator trim value: ");
      Serial.println(myData.ElevTrim);    
      Serial.print("Rudder trim value: ");
      Serial.println(myData.RudTrim);  
      Serial.print("Aileron trim value: ");
      Serial.println(myData.AilTrim); 


  // Send data using RF 433 MHz
  memcpy(tx_buf, &myData, sizeof(myData) );
  byte zize=sizeof(myData);
  rf_driver.send((uint8_t *)tx_buf, zize);  
  // rf_driver.send((uint8_t *)&myData, sizeof(myData));
  rf_driver.waitPacketSent();


    delay(1000);
}
